Imports System.Threading
Public MustInherit Class ClsBaseThread
    Implements IDisposable
#Region "Attributs partags"
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Attribut de classe,  tester pour le StopAllThread
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	01/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend Shared msCanRun As Boolean
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Mmorise le nombre de thread actuellement lancs
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend  Shared msNbThread As Integer
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Attribut de la classe, si true, arrt rapide des thread prfrable
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend  Shared msFinalize As Boolean
#End Region
#Region "Attributs protgs"
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Attribut d'objet  tester pour le StopThread
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	01/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend  mCanRun As Boolean = True
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Attribut d'objet true si le thread fonctionne (ne pas modifier, modification automatique)
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	01/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend  mThreadRun As Boolean = False
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Si ture, indique que l'arrt du thread doit tre rapide (gnration d'un ThreadAbortException toutes les 2 secondes)
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	01/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend  mFinalize As Boolean = False
#End Region
    Private mThread As Thread

    Shared Sub New()
        msNbThread = 0
        msFinalize = False
        msCanRun = True
    End Sub
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Constructeur de la classe+execution immdiate du thread
    ''' </summary>
    ''' <param name="pName">Optionnel, nom du thread</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	01/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend  Sub New(Optional ByVal pName As String = "")
#If DEBUG Then
        Debug.WriteLine("Dmarrage du thread : " + pName)
#End If
        mThread = New Thread(AddressOf SubThread)
        mThread.Name = pName
        mThread.Start()
    End Sub
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Retourne le nom du thread actuel
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	01/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public ReadOnly Property Name() As String
        Get
            Return mThread.Name
        End Get
    End Property
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Retourne ture si le thread est encode en fonctionnement.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	29/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public ReadOnly Property ThreadRun() As Boolean
        Get
            Return mThreadRun
        End Get
    End Property
#Region "Procdure concernants tous les threads"
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Arrte tous les threads actifs
    ''' </summary>
    ''' <param name="pWait">si true, attent que tous les threads soient arrts</param>
    ''' <param name="pHurry">Demande aux thread de s'arrter rapidement</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    friend  Shared Sub StopAllThread(Optional ByVal pWait As Boolean = False, Optional ByVal pHurry As Boolean = False)
        msCanRun = False
        If pHurry Then msFinalize = True
        Do While (pWait And msNbThread > 0) : Thread.Sleep(0) : Loop
    End Sub
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Retourne le nombre de thread actuellement en fonctionnement
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
     friend  Shared Function NbThreadRun() As Integer
        Return msNbThread
    End Function
#End Region
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Arrte le thread courant
    ''' </summary>
    ''' <param name="pWait">si true, attent que le thread soit arrt</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    friend  Sub StopThread(Optional ByVal pWait As Boolean = False)
        mCanRun = False
        Do While (pWait And mThreadRun) : Thread.Sleep(0) : Loop
    End Sub
#Region "mthodes  substituer"
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Mthode du thread
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected MustOverride Sub Run()
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Appel en cas d'erreur dans le thread
    ''' </summary>
    ''' <param name="pEx"></param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected MustOverride Sub SubError(ByRef pEx As Exception)
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Appel pour terminer le thread
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected MustOverride Sub EndThread()
#End Region
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Mthode de gestion du thread
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Private Sub SubThread()
        Try
            mThreadRun = True
            msNbThread += 1
            Run()
        Catch ex As Exception
            Try
                SubError(ex)
            Catch ex1 As Exception
#If DEBUG Then
                Debug.WriteLine("Erreur dans la procdure de gestion des erreurs du thread " + mThread.Name + Environment.NewLine + ex.ToString + Environment.NewLine + "Erreur gnre :" + Environment.NewLine + ex1.ToString)
#End If
            End Try
        Finally
            Try
                EndThread()
            Catch ex As Exception
                Try
                    SubError(ex)
                Catch ex1 As Exception
#If DEBUG Then
                    Debug.WriteLine("Erreur dans la procdure de gestion des erreurs de la procdure de fin du thread" + mThread.Name + Environment.NewLine + ex.ToString + Environment.NewLine + "Erreur gnre :" + Environment.NewLine + ex1.ToString)
#End If
                End Try
            End Try
            msNbThread -= 1
            mThreadRun = False
            System.GC.Collect()
            System.GC.WaitForPendingFinalizers()
        End Try
    End Sub
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Met le thread en pause pendant pMilisec millisecondes
    ''' </summary>
    ''' <param name="pMilisec">pause en ms</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	03/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected friend  Sub Sleep(ByVal pMilisec As Integer)
        mThread.Sleep(pMilisec)
    End Sub
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Retourne une String reprsentant l'tat de l'objet
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	03/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public overrides Function ToString() As String
        Dim lStr As String = "ClsBaseThread : "
        lStr += " Attributs shared {"
        lStr += "msCanRun=" + CStr(msCanRun)
        lStr += ",msNbThread=" + CStr(msNbThread)
        lStr += ",msFinalize=" + CStr(msFinalize)
        lStr += "} Attributs protected {"
        lStr += "mCanRun=" + CStr(mCanRun)
        lStr += ",mThreadRun=" + CStr(mThreadRun)
        lStr += ",msFinalize=" + CStr(msFinalize)
        lStr += "}"
        Return lStr
    End Function
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Dcharge l'objet immdiatement
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Friend Sub Dispose() Implements System.IDisposable.Dispose
        mCanRun = False
        mFinalize = True
    End Sub
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' destructeur
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[nicolas]	02/12/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Protected Overrides Sub Finalize()
        mCanRun = False
        mFinalize = True
        Do While mThreadRun
            If mThread.Join(2000) = False Then
                mThread.Abort()
            End If
        Loop
        MyBase.Finalize()
    End Sub
End Class
