Public NotInheritable Class ClsCopy
    Inherits ClsBaseThread
    Implements ICopy

    Private mDriveDest As ClsDrive
    Private mFileSrc As NHenry.Scandir.TFile
    'Variable utilises pour le calcul du dbit
    Private mTotalFileSize As Long
    Private mElapsedFileSize As Long
    Private mStartTime As Integer

    Public Sub New(ByVal pFileSrc As NHenry.Scandir.TFile, ByVal pDest As ClsDrive)
        MyBase.New("Copie du fichier : " + pFileSrc.FileName)
        If pFileSrc.IsExist = False Then Throw New System.MissingFieldException("ClsCopy", "pFileSrc")
        If pDest Is Nothing Then Throw New System.MissingFieldException("ClsCopy", "pDest")
        mDriveDest = pDest
        mFileSrc = pFileSrc
        If mDriveDest.GetFreeSpace.CompareTo(Convert.ToUInt64(mFileSrc.Length)) < 0 Then Throw New InvalidOperationException("Impossible de copier """ + mFileSrc.FileName + """ sur le lecteur """ + mDriveDest.DriveLetter + """, espace disque insuffisant.")
        mDriveDest.Copy = Me
        MyBase.Sleep(100)
        MyBase.StopThread()
    End Sub

    Public ReadOnly Property Drive() As ClsDrive
        Get
            Return mDriveDest
        End Get
    End Property
    Public ReadOnly Property ActFileCopy() As NHenry.Scandir.TFile Implements ICopy.ActFileCopy
        Get
            SyncLock Me
                If MyBase.mThreadRun Then
                    Return mFileSrc
                Else
                    Return New NHenry.Scandir.TFile("Erreur")
                End If
            End SyncLock
        End Get
    End Property

    Public ReadOnly Property TotalFileSize() As Long
        Get
            If mFileSrc.IsExist = False Then
                Return 0
            Else
                Return mFileSrc.Length
            End If
        End Get
    End Property
    Public ReadOnly Property ElapsedFileSize() As Long
        Get
            If mFileSrc.IsExist = False Then
                Return 0
            Else
                Return mElapsedFileSize
            End If
        End Get
    End Property
    Public ReadOnly Property Debit() As Long
        Get
            If mFileSrc.IsExist = False Then
                Return 0
            Else
                If mStartTime - Environment.TickCount = 0 Or mElapsedFileSize = 0 Then
                    Return 0
                Else
                    Dim lFreq As Long
                    Return CLng(mElapsedFileSize * 1000 / (Environment.TickCount - mStartTime))
                End If
            End If
        End Get
    End Property

    Public Sub MoveFile()
        'le buffer qui contiendra les donnes lues/crites
        Dim Buffer() As Byte
        Dim BufferCache As Integer
        Dim lDestFile As String = mDriveDest.DriveLetter.Substring(0, 1) + mFileSrc.FileName.Substring(1)
        MakeSurePath(lDestFile.Substring(0, lDestFile.LastIndexOf("\".ToCharArray)).TrimEnd("\".ToCharArray))

        ' La taille du buffer
        'BufferCache = 512
        BufferCache = mDriveDest.GetClusterSize() * 4
        'Do While BufferCache < 8000
        'BufferCache *= 2
        'Loop
        ReDim Buffer(BufferCache - 1)

        'stream des fichiers
        Dim FicSrc As New System.IO.FileStream(mFileSrc.FileName, IO.FileMode.Open)
        Dim FicSrcStream As New System.IO.BinaryReader(FicSrc)
        Dim FicDst As New System.IO.FileStream(lDestFile, IO.FileMode.Create)
        Dim FicDstStream As New System.IO.BinaryWriter(FicDst)

        ' La taille du fichier source
        mTotalFileSize = FicSrcStream.BaseStream.Length
        mElapsedFileSize = 0

        mStartTime = Environment.TickCount
        'Ici commence la boucle qui va copier le fichier
        Do
            'Si la taille restant  copier est infrieure  la
            'taille du buffer (fin de fichier), on "adapte" ce dernier
            If (mTotalFileSize - mElapsedFileSize) < BufferCache Then
                BufferCache = CInt((mTotalFileSize - mElapsedFileSize))
                ReDim Buffer(BufferCache - 1)
            End If
            ' On lit dans la source
            Buffer = FicSrcStream.ReadBytes(BufferCache)
            ' Calcul de ce qui reste  copier
            mElapsedFileSize += BufferCache
            ' On crire dans la destination
            FicDstStream.Write(Buffer)
            FicDstStream.Flush()
        Loop While (mTotalFileSize > mElapsedFileSize)
        ' On boucle tant qu'il le faut, 
        ' Soit tant que la taille restant  copier n'est pas nulle

        ' On ferme les fichiers
        FicSrcStream.Close()
        FicSrc.Close()
        FicDstStream.Close()
        FicDst.Close()

        'Suppression su fichier source
        System.IO.File.Delete(mFileSrc.FileName)
    End Sub
    Private Sub MakeSurePath(ByVal pPath As String)
        If pPath.Length <= 3 Then Exit Sub
        Dim lParent As String = pPath.Substring(0, pPath.LastIndexOf("\".ToCharArray)).TrimEnd("\".ToCharArray)
        If (New System.IO.DirectoryInfo(lParent)).Exists = False Then
            MakeSurePath(lParent)
        End If
        If (New System.IO.DirectoryInfo(lParent)).Exists = False Then
            Exit Sub
        End If
        Try
            Dim lPath As System.IO.DirectoryInfo = New System.IO.DirectoryInfo(lParent)
            lPath.CreateSubdirectory((New System.IO.DirectoryInfo(pPath)).Name)
        Catch ex As Exception
        End Try
    End Sub


    Protected Overrides Sub Run()
        Try
            Do While (MyBase.mCanRun And MyBase.msCanRun) AndAlso (mDriveDest Is Nothing OrElse mFileSrc.FileName <> "" OrElse mFileSrc.IsExist = False)
                MyBase.Sleep(100)
            Loop
            Dim lFile As NHenry.Scandir.TFile = mFileSrc
            If lFile.IsExist Then
                If Convert.ToUInt64(lFile.Length).CompareTo(mDriveDest.GetFreeSpace()) <= 0 Then
                    MoveFile()
                End If
            End If
        Catch ex As Exception
            Dim lCol As NHenry.Exception.ColParams = MyBase.GetErrParam(Nothing)
            With lCol
                .Add("Function", "ClsCopy.Run")
                .Add("mFileSrc", mFileSrc)
                .Add("FileName", mFileSrc.FileName)
                .Add("mDriveDest", mDriveDest)
                .Add("mTotalFileSize", mTotalFileSize)
                .Add("mElapsedFileSize", mElapsedFileSize)
                .Add("mStartTime", mStartTime)
                .Add("TickCount", Environment.TickCount)
            End With
            If NHenry.Exception.Main.Show(ex, _
            "Erreur dans le thread " + mDriveDest.DriveLetter, _
            "La copie du fichier """ + mDriveDest.DriveLetter.Substring(0, 1) + mFileSrc.FileName.Substring(1) + """ est peut-tre incomplte, voulez-vous supprimer le fichier copi ?", _
            lCol, _
            "Si ce problme persiste, vous pouvez contacter l'auteur de ce programme  l'adresse nhprog@laposte.net", _
            MessageBoxIcon.Question, _
            MessageBoxButtons.YesNo) _
            = DialogResult.Yes Then
                If IO.File.Exists(mDriveDest.DriveLetter.Substring(0, 1) + mFileSrc.FileName.Substring(1)) Then
                    System.GC.Collect()
                    System.GC.WaitForPendingFinalizers()
                    Try
                        IO.File.Delete(mDriveDest.DriveLetter.Substring(0, 1) + mFileSrc.FileName.Substring(1))
                    Catch ex1 As Exception
                        lCol.Add("ex", ex)
                        NHenry.Exception.Main.Show(ex1, _
                        "Erreur dans le thread " + mDriveDest.DriveLetter, _
                        "Erreur lors de la suppression du fichier """ + mDriveDest.DriveLetter.Substring(0, 1) + mFileSrc.FileName.Substring(1) + """", _
                        lCol, _
                        "Si ce problme persiste, vous pouvez contacter l'auteur de ce programme  l'adresse nhprog@laposte.net", _
                        MessageBoxIcon.Exclamation, _
                        MessageBoxButtons.OK)
                    End Try
                End If
            End If
        End Try
    End Sub
    Protected Overrides Sub EndThread()
        SyncLock Me
            mFileSrc.FileName = "Erreur"
            mDriveDest.Copy = Nothing
            mDriveDest = Nothing
        End SyncLock
    End Sub
    Protected Overrides Sub SubError(ByRef pEx As System.Exception)
        Dim lCol As NHenry.Exception.ColParams = MyBase.GetErrParam()
        lCol.Add("Me", Me)
        lCol.Add("mFileSrc", mFileSrc)
        lCol.Add("mDriveDest", mDriveDest)
        NHenry.Exception.Main.Show(pEx, "Erreur dans le thread : " + MyBase.Name, , lCol, , MessageBoxIcon.Exclamation, MessageBoxButtons.OK)
    End Sub

End Class
