Public NotInheritable Class ClsDrive
    Inherits ClsBaseThread

#Region "API"
    Private Declare Function GetDiskCluster Lib "kernel32" Alias "GetDiskFreeSpaceA" (ByVal lpRootPathName As String, ByRef lpSectorsPerCluster As Integer, ByRef lpBytesPerSector As Integer, ByRef lpNumberOfFreeClusters As Integer, ByRef lpTotalNumberOfClusters As Integer) As Integer
#End Region
    Private mDriveLetter As String = ""
    Private mDriveSpace As UInt64
    'Private mActFile As String = ""
    Private mLastRefreshSize As Integer = 0
    Private mDisk As System.Management.ManagementObject = Nothing

    Private mCopy As ICopy = Nothing

    'Variable utilises pour la gestion des copies
    Private mList As SortedList
    Private mLastCount As Integer

    Public Sub New(ByVal pDrive As String, ByRef pList As SortedList)
        MyBase.New("Gestion du lecteur " + pDrive)
        pDrive = pDrive.Replace("\", "")
        If Not (pDrive Like "?:") Then
            MyBase.mCanRun = False
            mDriveLetter = ""
            Exit Sub
        End If
        mDriveLetter = pDrive
        mList = pList
        mLastCount = 0
        RefreshSize()
    End Sub

    Public ReadOnly Property DriveLetter() As String
        Get
            Return mDriveLetter
        End Get
    End Property
    Public ReadOnly Property FileToMove() As String
        Get
            If mDriveLetter = "" Then Return ""
            If mCopy Is Nothing Then
                Return ""
            Else
                Return mCopy.ActFileCopy.FileTitle
            End If
        End Get
    End Property

    Public Property Copy() As ICopy
        Get
            Return mCopy
        End Get
        Set(ByVal Value As ICopy)
            mCopy = Value
        End Set
    End Property

    Public Overrides Function ToString() As String
        If MyBase.mThreadRun = False Or mDriveLetter = "" Then
            Return "!Error"
        End If
        'RefreshSize(False)
        If mCopy Is Nothing Then
            Return mDriveLetter + " " + GetSize(mDriveSpace)
        Else
            Return mDriveLetter + " " + GetSize(mDriveSpace) + " " + mCopy.ActFileCopy.FileTitle
        End If
    End Function

    Public Sub RefreshSize(Optional ByVal pForce As Boolean = True)
        SyncLock Me
            If pForce OrElse mLastRefreshSize + 3000 < Environment.TickCount OrElse mLastRefreshSize > Environment.TickCount Then
                GetDiskSpace()
                mLastRefreshSize = Environment.TickCount
            End If
        End SyncLock
        'MyBase.Sleep(0)
    End Sub
    Private Function GetDiskSpace() As System.UInt64
        SyncLock Me
            If mDriveLetter = "" Then Return Convert.ToUInt64(0)
            Dim space As System.UInt64
            If mDisk Is Nothing Then
                Dim diskClass As New System.Management.ManagementClass("Win32_LogicalDisk")
                Dim disks As System.Management.ManagementObjectCollection = diskClass.GetInstances()
                Dim disk As System.Management.ManagementObject
                For Each disk In disks
                    If CStr(disk("Name")) = mDriveLetter Then
                        mDisk = disk
                        Exit For
                    End If
                Next disk
            End If
            If Not (mDisk Is Nothing) Then
                mDisk.Get()
                space = CType(mDisk("FreeSpace"), System.UInt64)
                Dim lTest As String = CType(mDisk("Name"), String)
                Me.mDriveSpace = space
            Else
                Me.mDriveSpace = Convert.ToUInt64(0)
                space = Convert.ToUInt64(0)
            End If
            Return space
        End SyncLock
    End Function
    Public Function GetFreeSpace(Optional ByVal pRefreshSize As Boolean = True) As UInt64
        If pRefreshSize Then RefreshSize()
        Return mDriveSpace
    End Function

    Public Function GetClusterSize(Optional ByVal pRoot As String = "") As Integer
        If mDriveLetter.Length < 2 And pRoot = "" Then Return 0
        Dim lRoot As String
        If pRoot = "" Then
            lRoot = mDriveLetter.Substring(0, 2)
        Else
            lRoot = pRoot
        End If
        Dim lSectorPerCluster As Integer
        Dim lBytePerSector As Integer
        Dim lFreeCluster As Integer
        Dim lTotalCluster As Integer
        GetDiskCluster(lRoot + vbNullChar, lSectorPerCluster, lBytePerSector, lFreeCluster, lTotalCluster)
        Return lSectorPerCluster * lBytePerSector
    End Function

    Protected Overrides Sub Run()
        Dim lDestFile As String
        Dim lRet As Long
        Do While MyBase.mCanRun And MyBase.msCanRun
            If mDriveLetter <> "" Then
                RefreshSize(False)
            End If
            If mList.Count <> mLastCount AndAlso mList.Count > 0 AndAlso mCopy Is Nothing Then
                Dim lFile As NHenry.Scandir.TFile
                lFile.FileName = "Vide"
                RefreshSize()
                SyncLock mList
                    For i As Integer = mList.Count - 1 To 0 Step -1
                        Dim lTmpFile As NHenry.Scandir.TFile = DirectCast(mList.GetByIndex(i), NHenry.Scandir.TFile)
                        If lTmpFile.Length <= Convert.ToInt64(Me.mDriveSpace) Then
                            lFile = lTmpFile
                            mList.RemoveAt(i)
                            Exit For
                        End If
                    Next
                End SyncLock
                Try
                    If lFile.IsExist Then
                        Dim lThread As New ClsCopy(lFile, Me)
                    Else
                        mLastCount = mList.Count
                    End If
                Catch ex As Exception
                    mlastcount = mList.Count
                End Try
                MyBase.Sleep(500)
            ElseIf mCopy Is Nothing Then
                mLastCount = mList.Count
                MyBase.Sleep(500)
            Else
                If mCopy.ActFileCopy.IsCorrect = False Then mCopy = Nothing
                MyBase.Sleep(100)
            End If
        Loop
    End Sub
    Protected Overrides Sub EndThread()
        mDriveLetter = ""
    End Sub
    Protected Overrides Sub SubError(ByRef pEx As System.Exception)
#If DEBUG Then
        Debug.WriteLine(pEx.ToString)
#End If
    End Sub

#Region "Shared methodes"
    Public Shared Function GetSize(ByVal pSize As UInt64) As String
        Dim i As Integer = 0
        Dim lStr As String

        Do While (Convert.ToDouble(pSize) / (1024 ^ i)) > 1024
            i += 1
        Loop

        If pSize.Equals(Convert.ToUInt64(0)) Then
            lStr = "0"
        Else
            lStr = Format((Convert.ToDouble(pSize) / (1024 ^ i)), "#.##")
        End If
        Select Case i
            Case 0
                lStr += " o"
            Case 1
                lStr += " ko"
            Case 2
                lStr += " Mo"
            Case 3
                lStr += " Go"
            Case 4
                lStr += " To"
            Case 5
                lStr += " Po"
            Case Else
                lStr += " " + CStr(i)
        End Select
        Return lStr
    End Function
    Public Shared Function GetSize(ByVal pSize As Long) As String
        Dim i As Integer = 0
        Dim lStr As String

        Do While (Convert.ToDouble(pSize) / (1024 ^ i)) > 1024
            i += 1
        Loop

        If pSize = 0 Then
            lStr = "0"
        Else
            lStr = Format((Convert.ToDouble(pSize) / (1024 ^ i)), "#.##")
        End If
        Select Case i
            Case 0
                lStr += " o"
            Case 1
                lStr += " ko"
            Case 2
                lStr += " Mo"
            Case 3
                lStr += " Go"
            Case 4
                lStr += " To"
            Case 5
                lStr += " Po"
            Case Else
                lStr += " " + CStr(i)
        End Select
        Return lStr
    End Function
    Public Shared Function GetSize(ByVal pSize As Integer) As String
        Dim i As Integer = 0
        Dim lStr As String

        Do While (Convert.ToDouble(pSize) / (1024 ^ i)) > 1024
            i += 1
        Loop

        If pSize = 0 Then
            lStr = "0"
        Else
            lStr = Format((Convert.ToDouble(pSize) / (1024 ^ i)), "#.##")
        End If
        Select Case i
            Case 0
                lStr += " o"
            Case 1
                lStr += " ko"
            Case 2
                lStr += " Mo"
            Case 3
                lStr += " Go"
            Case 4
                lStr += " To"
            Case 5
                lStr += " Po"
            Case Else
                lStr += " 2^" + CStr(i)
        End Select
        Return lStr
    End Function
    Public Shared Function GetNbCopy() As Integer
        Dim i As Integer = 0
        Dim lTh As ClsBaseThread
        For Each lTh In msLstThreads
            If TypeOf lTh Is ClsDrive Then
                If Not (CType(lTh, ClsDrive).mCopy Is Nothing) Then
                    i += 1
                End If
            End If
        Next
        Return i
    End Function
    Public Shared Function GetFirstCopy(ByVal pFileSize As Long) As ClsDrive
        Dim lCanTh As ClsDrive
        Dim lTh As ClsBaseThread
        For Each lTh In msLstThreads
            If TypeOf lTh Is ClsDrive Then
                If Convert.ToInt64(CType(lTh, ClsDrive).GetFreeSpace) > pFileSize Then
                    If CType(lTh, ClsDrive).mCopy Is Nothing Then
                        lCanTh = CType(lTh, ClsDrive)
                        Return lCanTh
                    ElseIf lCanTh Is Nothing Then
                        lCanTh = CType(lTh, ClsDrive)
                    End If
                End If
            End If
        Next
        Return lCanTh
    End Function
#End Region
End Class
